from typing import Generic, TypeVar

from rainbowadn.hashing.hashmentionable import HashMentionable
from rainbowadn.hashing.hashpoint import HashPoint
from rainbowadn.hashing.hashresolver import HashResolver
from rainbowadn.hashing.origin import Origin
from rainbowadn.hashing.rainbow_factory import RainbowFactory

__all__ = ('ResolverOrigin',)

OriginType = TypeVar('OriginType')


class ResolverOrigin(Origin[OriginType], Generic[OriginType]):
    def __init__(
            self,
            factory: RainbowFactory[OriginType],
            point: bytes,
            resolver: HashResolver
    ):
        assert isinstance(factory, RainbowFactory)
        assert isinstance(point, bytes)
        assert isinstance(resolver, HashResolver)
        self.factory = factory
        self.point = point
        self.resolver = resolver

    def resolve(self) -> OriginType:
        resolved: bytes = self.resolver.resolve_bytes(self.point)
        assert isinstance(resolved, bytes)
        mentioned: OriginType = self.factory.from_bytes(resolved[HashPoint.HASH_LENGTH:], self.resolver)
        assert isinstance(mentioned, HashMentionable)
        assert mentioned.__topology_hash__() == resolved[:HashPoint.HASH_LENGTH]
        return mentioned

    def hash_point(self) -> HashPoint[OriginType]:
        return HashPoint(self.factory, self.point, self)
